// Copyright 2005-2007 Opsware, Inc.  All rights reserved.

import com.opsware.client.*;
import com.opsware.server.*;
import com.opsware.search.*;
import com.opsware.job.*;
import com.opsware.swmgmt.*; 
import com.opsware.common.*; 
import java.lang.reflect.*;
import java.util.*;
import java.text.*;
import java.io.*;

class GetRemediationJobResult {

    // Default host name of the Opsware SAS core server.
    private static String host = "occ";

    // Default port of where the Opsware server is listenting for requests
    private static int port = 443;

    // The Opsware ID of the remediation job.
    private static Long jobID;

    // A handle to the JobService.
    private static JobService jobSvc;

    // A reference to the remediation job.
    private static JobRef jobRef;


    public static void main(String[] args) {
        if (!parseArgs(args)) {
            System.exit(1);
            return;
        }
        try {
            login();
            getJobInfo();
            getRemediationJobResult();

        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
            System.exit(1);
        }
    }


    protected static boolean parseArgs(String[] args) {
        // Parse the command-line arguments.
        for (int i=0; i < args.length; i++) {
            if ("--host".equals(args[i])) {
                host = args[++i];

            } else if ("--port".equals(args[i])) {
                port = java.lang.Integer.parseInt(args[++i]);

            } else {
                jobID = java.lang.Long.valueOf(args[i]);
              
            }
        }

        if (jobID == null) {
            System.out.println("Missing job ID.");
            usage();
            return false;
        }

        return true;
    }

    protected static void usage() {
        StringBuffer sbuf = new StringBuffer();
        sbuf.append("usage: GetServerInfo [options] <target>\n");
        sbuf.append("option --host <occ host>   OCC host name.\n");
        sbuf.append("option --port <occ port>   OCC port.\n");
        sbuf.append("<jobID> ID of remediation job that has completed.\n");
        System.out.println(sbuf.toString());
    }

    protected static void login() throws Exception {
        // Prompt for the user name and password.
        String[] userPasswd = getUserPassword();

        // Use the OpswareClient to create an https connection to Opsware SAS
        // and authenticate the user.
        OpswareClient.connect("https", host, (short)port, userPasswd[0],
                              userPasswd[1], true);

        // Get the handle to the required service.
        jobSvc = (JobService)OpswareClient.getService(JobService.class);
    }

    protected static String[] getUserPassword() throws IOException {
        // Prompt for the user name and password.
        String[] userPasswd = new String[2];
        BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
        System.out.print("Enter Opsware user name: ");
        userPasswd[0] = reader.readLine();

        System.out.print("Enter Opsware user password: ");
        userPasswd[1] = reader.readLine();

        return userPasswd;
    }

    private static String jobStatusToString(int status) {
        String statusString;
        switch (status) {
           case JobInfoVO.STATUS_ABORTED: statusString = "ABORTED" ; break;
           case JobInfoVO.STATUS_ACTIVE: statusString = "ACTIVE" ; break;
           case JobInfoVO.STATUS_BLOCKED: statusString = "BLOCKED" ; break;
           case JobInfoVO.STATUS_CANCELLED: statusString = "CANCELLED" ; break;
           case JobInfoVO.STATUS_DELETED: statusString = "DELETED" ; break;
           case JobInfoVO.STATUS_EXPIRED: statusString = "EXPIRED" ; break;
           case JobInfoVO.STATUS_FAILURE: statusString = "FAILURE" ; break;
           case JobInfoVO.STATUS_PENDING: statusString = "PENDING" ; break;
           case JobInfoVO.STATUS_RECURRING: statusString = "RECURRING" ; break;
           case JobInfoVO.STATUS_STALE : statusString = "STALE" ; break;
           case JobInfoVO.STATUS_SUCCESS: statusString = "SUCCESS" ; break;
           case JobInfoVO.STATUS_TAMPERED: statusString = "TAMPERED" ; break;
           case JobInfoVO.STATUS_UNKNOWN: statusString = "UNKNOWN" ; break;
           case JobInfoVO.STATUS_WARNING: statusString = "WARNING" ; break;
           case JobInfoVO.STATUS_ZOMBIE : statusString = "ZOMBIE" ; break;
           default: statusString = "UNKNOWN" ; break;
        } 
     return statusString;
    } 
 
    private static void printDate(String label, Date date) {
       String formatString = "yyyy.MM.dd.HH.mm.ss";
       Format formatter = new SimpleDateFormat(formatString);
        if (date == null) {
           System.out.println (label + "null");
        }
        else {
           System.out.println(label + formatter.format(date) +
                " (" + formatString + ")" );
        }
    } 


    protected static void getJobInfo() throws Exception {

       // Get the value object (VO) of the specified job ID.
       jobRef = new JobRef(jobID);
       JobInfoVO jobInfoVO = null;
       try {
          jobInfoVO = jobSvc.getJobInfoVO(jobRef);
       } catch (NotFoundException e) {
          System.out.println("Job with ID of " + jobID + " not found.");
          System.exit(1);
       }

       System.out.println ("Information from JobInfoVO:");
       System.out.println ("   Description: " + jobInfoVO.getDescription());
       System.out.println ("   Type: " + jobInfoVO.getType());
       System.out.println ("   UserName: " + jobInfoVO.getUserName());

       int jobStatus = jobInfoVO.getStatus();
       System.out.println ("   Status: " + jobStatusToString(jobStatus));
       printDate("   startDate: ", jobInfoVO.getStartDate());
       printDate("   endDate: ", jobInfoVO.getEndDate());
       System.out.println ();

       // If the job is still running (active), exit this program.
       // There are no job results for an active job.
       if (jobStatus == JobInfoVO.STATUS_ACTIVE) {
          System.out.println
             ("The specified job is still running, but this program");
          System.out.println
             ("is only smart enough to examine the results of a job");
          System.out.println
             ("that has completed.  Exiting now....");
          System.exit(0);
       } 
    }

    protected static void getRemediationJobResult() throws Exception {

       JobResult jobResult = 
          jobSvc.getResult(jobRef);
       Class jobResultClass = jobResult.getClass();

       // If this is not a remediation job, exit this program.
       // The subclasses of JobResult have different attributes.
       // This program is limited to examining the attributes of a
       // of a remediation job.
       if (jobResultClass != com.opsware.swmgmt.RemediateJobResult.class) {
          System.out.println
             ("The specified job is not a remediation job.");
          System.out.println ("JobResult class = " + jobResultClass.getName());
          System.out.println
             ("Exiting now....");
          System.exit(0);
       }

       RemediateJobResult remJobResult = (RemediateJobResult)jobResult;

       // JobElementResult[] contains information that is displayed in the
       // table on the Job Status window of the SAS Client.
       JobElementResult[] jobElementResult = remJobResult.getElemResultInfo();

       System.out.println("Information from JobElementResult array:");
       System.out.println();

       // Iterate through the JobElementResult array and print out
       // some of the attributes.
       for (int i = 0; i < jobElementResult.length; i++) {

          JobElementResult subresult = jobElementResult[i];
          Class jobElemClass = subresult.getClass();
          System.out.println 
             ("JobElementResult subclass: " + jobElemClass.getName());

          // JobElementResult array can contain different types of
          // classes, which have different types of attributes.
          // Therefore, the specific subclass is determined before
          // getting and printing its attributes.

          if (jobElemClass == com.opsware.swmgmt.PhaseJobResult.class) {
              PhaseJobResult pres = (PhaseJobResult)subresult;
              System.out.println 
                 ("   phaseType: " + pres.getPhaseType());
              printDate("   startDate: ", pres.getStartDate());
              printDate("   endDate: ", pres.getEndDate());
          }
          else if (jobElemClass == com.opsware.swmgmt.DeviceJobResult.class) {
              DeviceJobResult dres = (DeviceJobResult)subresult;
              System.out.println 
                 ("   phase: " + dres.getPhase());
              ServerRef serverRef = (ServerRef)subresult.getElement();
              System.out.println 
                 ("   Server Id: " + serverRef.getId());
          }
          else if (jobElemClass == com.opsware.swmgmt.ScriptJobResult.class) {
              ScriptJobResult sres = (ScriptJobResult)subresult;
              System.out.println 
                 ("   action: " + sres.getAction());
              System.out.println 
                 ("   scriptName: " + sres.getScriptName());
              System.out.println 
                 ("   status: " + sres.getStatus());
              ServerRef serverRef = (ServerRef)subresult.getElement();
              System.out.println 
                 ("   Server Id: " + serverRef.getId());
          }
          else if (jobElemClass == com.opsware.swmgmt.UnitJobResult.class) {
              UnitJobResult ures = (UnitJobResult)subresult;
              System.out.println 
                 ("   action: " + ures.getAction());
              System.out.println 
                 ("   status: " + ures.getStatus());
              System.out.println 
                 ("   unitName: " + ures.getUnitName());
              ServerRef serverRef = (ServerRef)subresult.getElement();
              System.out.println 
                 ("   Server Id: " + serverRef.getId());
              JobMessageInfo[] messageInfos = ures.getMessageInfos();
              for (int k = 0; k < messageInfos.length; k++) {
                 System.out.println 
                    ("   messageInfos: " + messageInfos[k].getKey());
              } 
          }
          else {
            System.out.println 
               ("No information is printed for this JobElementResult.");
          }
       } // for
   } 

} 

