// Copyright 2005-2007 Opsware, Inc.  All rights reserved.

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;
import java.util.StringTokenizer;

import com.opsware.client.OpswClientEndpoint;
import com.opsware.client.OpswareClient;
import com.opsware.common.NotFoundException;
import com.opsware.device.PlatformRef;
import com.opsware.folder.FolderRef;
import com.opsware.folder.FolderService;
import com.opsware.folder.FolderVO;
import com.opsware.job.JobInfoVO;
import com.opsware.job.JobProgress;
import com.opsware.job.JobRef;
import com.opsware.job.JobResult;
import com.opsware.job.JobService;
import com.opsware.locality.CustomerRef;
import com.opsware.pkg.UnitReference;
import com.opsware.pkg.UnitService;
import com.opsware.pkg.UnitType;
import com.opsware.pkg.ZIPRef;
import com.opsware.pkg.ZIPService;
import com.opsware.pkg.ZIPVO;
import com.opsware.search.Filter;
import com.opsware.server.ServerRef;
import com.opsware.server.ServerService;
import com.opsware.server.ServerVO;
import com.opsware.swmgmt.SoftwarePolicyItemReference;
import com.opsware.swmgmt.SoftwarePolicyRef;
import com.opsware.swmgmt.SoftwarePolicyService;
import com.opsware.swmgmt.SoftwarePolicyVO;
import com.opsware.unitio.StreamedUnitImpl;
import com.opsware.unitio.UnitCreationParams;
import com.opsware.unitio.UnitInputStream;
import com.opsware.unitio.UnitInputStreamEvent;
import com.opsware.unitio.UnitInputStreamListener;

/**
 * A simple command-line application that installs a given package across a list of servers
 * managed by Opsware.
 * 
 */
public class PackageInstaller {

    /**
     * Top level folder to contain all adhoc policies created by PackageInstaller
     */
    private final static String[]        FOLDER    = { "pkg installer policies" };

    /**
     * List of user specified target host names.
     */
    private static LinkedList            hostNames = new LinkedList();

    /**
     * Installation path of the zip package
     */
    private static String                path      = null;

    /**
     * The package to install on the target hosts.
     */
    private static File                  pkg;

    /**
     * Default host name of the Opsware server
     */
    private static String                host      = "occ";

    /**
     * Default port of where the Opsware server is listenting for requests
     */
    private static int                   port      = 443;

    /**
     * A handle to the SoftwarePolicyService
     */
    private static SoftwarePolicyService appPolicySvc;

    /**
     * A handle to the ServerService
     */
    private static ServerService         serverSvc;

    /**
     * A handle to the FolderService
     */
    private static FolderService         folderSvc;

    /**
     * A handle to the JobService
     */
    private static JobService            jobSvc;

    /**
     * A handle to the UnitService
     */
    private static UnitService           unitSvc;

    /**
     * A handle to the ZIPService
     */
    private static ZIPService            zipSvc;

    /**
     * Main
     */
    public static void main(String[] args) {
        if (!parseArgs(args)) {
            System.exit(1);
            return;
        }

        try {
            // 1. Connect and log into Opsware server
            login();

            // 2. Get server references to specified servers.
            ServerRef[] servers = getServerRefs(hostNames);

            // 3. Get a union of all platforms
            PlatformRef[] platforms = getPlatforms(servers);

            // 4. Find or Create folder
            FolderRef folder = getFolder();

            // 5. Upload package into folder
            UnitReference[] units = upload(folder, platforms);

            // 6. Create policy
            SoftwarePolicyVO policy = createPolicy(folder, units, platforms);
            SoftwarePolicyRef pRef = (SoftwarePolicyRef) policy.getRef();

            // 7. Associate policy with server
            appPolicySvc.attachToPolicies(new SoftwarePolicyRef[] { pRef }, servers);

            // 8. Put the package on the servers
            partialReconcile(pRef, servers);

        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
            System.exit(1);
        }
    }

    protected static boolean parseArgs(String[] args) {
        String file = null;
        String servers = null;

        // parse args
        for (int i = 0; i < args.length; i++) {
            if ("--host".equals(args[i])) {
                host = args[++i];

            } else if ("--port".equals(args[i])) {
                port = Integer.parseInt(args[++i]);

            } else if ("--path".equals(args[i])) {
                path = args[++i];

            } else if (file == null) {
                file = args[i];

            } else {
                servers = args[i];
            }
        }

        if (file == null) {
            System.out.println("Missing file package.");
            usage();
            return false;
        }

        if (servers == null) {
            System.out.println("Missing target hosts.");
            usage();
            return false;
        }

        for (StringTokenizer strtok = new StringTokenizer(servers, ","); strtok.hasMoreTokens();) {
            hostNames.add(strtok.nextToken());
        }

        pkg = new File(file);
        if (!pkg.exists()) {
            System.out.println("Package " + file + ", does not exist!");
            usage();
            return false;
        }
        return true;
    }

    protected static void usage() {
        StringBuffer sbuf = new StringBuffer();
        sbuf.append("usage: installpkg [options] <file pkg> <hostname>[,<hostname>]\n");
        sbuf.append("option --host <occ host>   OCC host name.\n");
        sbuf.append("option --port <occ port>   OCC port.\n");
        sbuf.append("option --path <install path>  Package installation path\n");
        System.out.println(sbuf.toString());
    }

    protected static void login() throws Exception {
        String[] userPasswd = getUserPassword();

        // Use the OpswareClient to create an https connection to Opsware and
        // authenticate the user.
        OpswareClient.connect("https", host, (short) port, userPasswd[0], userPasswd[1], true);

        // Get handle to required services
        appPolicySvc = (SoftwarePolicyService) OpswareClient.getService(SoftwarePolicyService.class);
        serverSvc = (ServerService) OpswareClient.getService(ServerService.class);
        folderSvc = (FolderService) OpswareClient.getService(FolderService.class);
        jobSvc = (JobService) OpswareClient.getService(JobService.class);
        unitSvc = (UnitService) OpswareClient.getService(UnitService.class);
        zipSvc = (ZIPService) OpswareClient.getService(ZIPService.class);
    }

    protected static void partialReconcile(SoftwarePolicyRef policy, ServerRef[] servers) throws Exception {
        SoftwarePolicyRef[] policies = new SoftwarePolicyRef[] { policy };
        JobRef job = appPolicySvc.startRemediateNow(policies, servers);

        System.out.print(".");
        Thread.sleep(10 * 1000);
        JobProgress progress = jobSvc.getProgress(job);
        while (progress != null) {
            System.out.print(".");
            Thread.sleep(10 * 1000); // poll every 10 seconds.
            progress = jobSvc.getProgress(job);
        }

        JobResult res = jobSvc.getResult(job);
        if (res.getJobInfo().getStatus() == JobInfoVO.STATUS_SUCCESS) {
            System.out.println("\nSuccesfully installed " + policy.getName());
        } else if (res.getError() != null) {
            throw res.getError();

        } else {
            System.out.println("\nError installing " + policy.getName() + "\nJob id: " + job);
        }
    }

    protected static SoftwarePolicyVO createPolicy(FolderRef folder, UnitReference[] units, PlatformRef[] platforms)
            throws Exception {
        SoftwarePolicyVO vo = new SoftwarePolicyVO();
        vo.setPlatforms(platforms);
        vo.setName(pkg.getName() + " " + Math.abs(new Random().nextInt() % 5000));
        vo.setFolder(folder);

        SoftwarePolicyItemReference[] items = new SoftwarePolicyItemReference[units.length];
        for (int i = 0; i < units.length; i++) {
            items[i] = (SoftwarePolicyItemReference) units[i];
        }
        vo.setSoftwarePolicyItems(items);
        vo.setDescription("Created by PackageInstaller.");
        return appPolicySvc.create(vo);
    }

    protected static UnitReference[] upload(FolderRef folder, PlatformRef[] platforms) throws Exception {
        UnitType unitType = unitSvc.getUnitType("ZIP");

        // Create input stream with listener
        UnitInputStream uis = new UnitInputStream(new FileInputStream(pkg), pkg.length());
        uis.addUnitStreamListener(new UnitInputStreamListener() {

            public void streamRead(UnitInputStreamEvent e) {
                if (!e.isComplete()) {
                    System.out.print(".");
                }
            }
        });

        // Create package parameters
        UnitCreationParams params = new UnitCreationParams(uis, pkg.getName(), unitType.getTypeName(), platforms,
                folder);

        // Create an package uploader
        String token = OpswClientEndpoint.getInst().getToken(false);
        StreamedUnitImpl uploader = new StreamedUnitImpl(token, "https", host, 443);

        // Upload the pacakge
        UnitReference[] uref = uploader.create(params);

        // Set installation paths
        if (path != null) {
            for (int i = 0; i < uref.length; i++) {
                ZIPRef ref = (ZIPRef) uref[i];
                ZIPVO vo = zipSvc.getZIPVO(ref);
                vo.setInstallPath(path);
                zipSvc.update(ref, vo, false, false);
            }
        }
        return uref;
    }

    protected static FolderRef getFolder() throws Exception {
        // First try to find the folder
        try {
            return folderSvc.getFolderRef(FOLDER);
        } catch (NotFoundException e) {
            // not found, continue and create folder.
        }

        // Create the folder
        FolderVO folderVo = folderSvc.createPath(folderSvc.getRoot(), FOLDER);
        FolderRef folder = (FolderRef) folderVo.getRef();
        folderSvc.setCustomers(folder, new CustomerRef[] { new CustomerRef(0) }, false);
        return folder;
    }

    protected static PlatformRef[] getPlatforms(ServerRef[] servers) throws Exception {
        ServerVO[] vos = serverSvc.getServerVOs(servers);
        PlatformRef[] platforms = new PlatformRef[vos.length];
        for (int i = 0; i < vos.length; i++) {
            platforms[i] = vos[i].getPlatform();
        }
        return platforms;
    }

    protected static ServerRef[] getServerRefs(LinkedList hosts) throws Exception {
        // Build search filter
        StringBuffer filterStr = new StringBuffer();
        for (Iterator itr = hosts.iterator(); itr.hasNext();) {
            filterStr.append("(ServerVO.hostName = \"").append(itr.next()).append("\")");
            if (itr.hasNext()) {
                filterStr.append("||");
            }
        }

        // Get Server refs
        return serverSvc.findServerRefs(new Filter(null, filterStr.toString()));
    }

    protected static String[] getUserPassword() throws IOException {
        String[] userPasswd = new String[2];
        BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
        System.out.print("Enter Opsware user name: ");
        userPasswd[0] = reader.readLine();

        System.out.print("Enter Opsware user password: ");
        userPasswd[1] = reader.readLine();

        return userPasswd;
    }
}
