// Copyright 2005-2007 Opsware, Inc.  All rights reserved.

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

import com.opsware.client.OpswareClient;
import com.opsware.common.NotFoundException;
import com.opsware.common.ObjectReference;
import com.opsware.compliance.sco.AuditTaskRef;
import com.opsware.compliance.sco.AuditTaskService;
import com.opsware.compliance.sco.AuditTaskTargetReference;
import com.opsware.compliance.sco.AuditTaskVO;
import com.opsware.device.DeviceGroupRef;
import com.opsware.device.DeviceGroupService;
import com.opsware.job.JobInfoVO;
import com.opsware.job.JobRef;
import com.opsware.job.JobService;
import com.opsware.search.Filter;
import com.opsware.server.ServerRef;
import com.opsware.server.ServerService;

/**
 * A simple command-line application that runs predefined audits against the
 * given server only.
 *
 */
public class RunAudits {

    /**
     * Default host name of the Opsware server
     */
    private static String host = "occ";

    /**
     * Default port of where the Opsware server is listenting for requests
     */
    private static int port = 443;

    /**
     * Target server hostname.
     */
    private static String server;

    /**
     * A handle to the AuditTaskService
     */
    private static AuditTaskService auditTaskSvc;

    /**
     * A handle to the ServerService
     */
    private static ServerService serverSvc;

    /**
     * A handle to the JobService
     */
    private static JobService jobSvc;

    /**
     * A handle to the DeviceGroupService
     */
    private static DeviceGroupService groupSvc;


    /**
     * Main
     */
    public static void main(String[] args) {
        if (!parseArgs(args)) {
            System.exit(1);
            return;
        }

        try {
            // 1. Connect and log into Opsware.
            login();

            // 2. Get server references to specified server.
            ServerRef serverRef = getServerRef(server);

            // 3. Get audits with given server as a target.
            AuditTaskVO[]audits = getAudits(serverRef);

            // 4. Run audits against the server
            runAudits(audits, serverRef);

        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
            System.exit(1);
        }
    }

    protected static boolean parseArgs(String[] args) {
        // parse args
        for (int i=0; i < args.length; i++) {
            if ("--host".equals(args[i])) {
                host = args[++i];

            } else if ("--port".equals(args[i])) {
                port = Integer.parseInt(args[++i]);

            } else {
                server = args[i];
            }
        }

        if (server == null) {
            System.out.println("Missing target server.");
            usage();
            return false;
        }

        return true;
    }

    protected static void usage() {
        StringBuffer sbuf = new StringBuffer();
        sbuf.append("usage: runaudits [options] <hostname>\n");
        sbuf.append("option --host <occ host>   OCC host name.\n");
        sbuf.append("option --port <occ port>   OCC port.\n");
        System.out.println(sbuf.toString());
    }

    protected static void login() throws Exception {
        String[] userPasswd = getUserPassword();

        // Use the OpswareClient to create an https connection to Opsware and
        // authenticate the user.
        OpswareClient.connect("https", host, (short)port, userPasswd[0],
                              userPasswd[1], true);

        // Get handle to required services
        auditTaskSvc = (AuditTaskService)
                       OpswareClient.getService(AuditTaskService.class);
        serverSvc = (ServerService)OpswareClient.getService(ServerService.class);
        jobSvc = (JobService)OpswareClient.getService(JobService.class);
        groupSvc = (DeviceGroupService)OpswareClient.getService(DeviceGroupService.class);
    }

    protected static void runAudits(AuditTaskVO[] audits, ServerRef server) throws Exception {
        LinkedList jobs = new LinkedList();
        for (int i=0; i < audits.length; i++) {
            String name = "ad hoc audit run: " + i;
            audits[i].setTargets(new AuditTaskTargetReference[] {server});
            audits[i].setName(name);
            jobs.add(auditTaskSvc.startAdHocAudit(audits[i], name, null, null));
        }
        waitForCompletion(jobs);
    }

    protected static void waitForCompletion(LinkedList jobs) throws Exception {
        StringBuffer msg = new StringBuffer("Waiting for audit jobs: ");
        for (Iterator itr=jobs.iterator(); itr.hasNext();) {
            msg.append(itr.next());
            if (itr.hasNext()) {
                msg.append(", ");
            }
        }
        System.out.println(msg.toString() + " to complete.");

        while (jobs.size() > 0) {
            System.out.print(".");
            Thread.sleep(10 * 1000);
            JobInfoVO[] stats = jobSvc.getJobInfoVOs((JobRef[])jobs.toArray(new JobRef[jobs.size()]));
            for (int i=0; i < stats.length; i++) {
                switch (stats[i].getStatus()) {
                    case JobInfoVO.STATUS_PENDING:
                    case JobInfoVO.STATUS_RECURRING:
                    case JobInfoVO.STATUS_BLOCKED:
                    case JobInfoVO.STATUS_ACTIVE:
                        break;
                    default:
                        jobs.remove(stats[i].getRef());
                }
            }
        }
        System.out.println("Done.");
    }

    protected static ServerRef getServerRef(String hostname)
        throws Exception {
        Filter filter = new Filter(null, "(ServerVO.hostName = \"" +
                                           hostname + "\")");
        // Get Server refs
        return serverSvc.findServerRefs(filter)[0];
    }

    protected static AuditTaskVO[] getAudits(ServerRef sref) throws Exception {
        AuditTaskRef[]aRefs = auditTaskSvc.findAuditTaskRefs(null);
        AuditTaskVO[] aVOs = auditTaskSvc.getAuditTaskVOs(aRefs);
        HashSet audits = new HashSet();
        for (int i=0; i < aVOs.length; i++) {
            HashSet srvrs = flattenTargets((ObjectReference[])aVOs[i].getTargets(), new HashSet());
            if (srvrs.contains(sref)) {
                audits.add(aVOs[i]);
            }
        }
        return (AuditTaskVO[])audits.toArray(new AuditTaskVO[audits.size()]);
    }

    protected static HashSet flattenTargets(ObjectReference[] targets, HashSet srefs)
        throws Exception {
        for (int i=0; i < targets.length; i++) {
            if (targets[i] instanceof ServerRef) {
                srefs.add(targets[i]);
            } else if (targets[i] instanceof DeviceGroupRef) {
                try {
                    flattenTargets((ObjectReference[])groupSvc.getDevices((DeviceGroupRef)targets[i]), srefs);
                    flattenTargets((ObjectReference[])groupSvc.getChildren((DeviceGroupRef)targets[i]), srefs);
                } catch (NotFoundException e) {
                    // skip groups no longer in the system
                }
            }
        }
        return srefs;
    }

    protected static String[] getUserPassword() throws IOException {
        String[] userPasswd = new String[2];
        BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
        System.out.print("Enter Opsware user name: ");
        userPasswd[0] = reader.readLine();

        System.out.print("Enter Opsware user password: ");
        userPasswd[1] = reader.readLine();

        return userPasswd;
    }
}
