// Copyright 2005-2007 Opsware, Inc.  All rights reserved.

import com.opsware.client.*;
import com.opsware.server.*;
import com.opsware.search.*;
import com.opsware.job.*;
import com.opsware.swmgmt.*;
import com.opsware.fido.AuthenticationException;
import java.util.*;
import java.text.*;
import java.io.*;

public class ScheduleRemediation {

    // Default host name of the Opsware SAS core server.
    private static String host = "occ";

    // Default port of where the Opsware server is listenting for requests
    private static int port = 443;

    // The Opsware ID of the software policy.
    private static long policyID;

    // The Opsware ID of the server to be remediated with the software policy.
    private static long serverID;

    // A handle to the SoftwarePolicyService.
    private static SoftwarePolicyService swpSvc;

    // A reference to the remediation job.
    private static JobRef jobRef;

    // The number of minutes from now when to run the stage
    // (download) phase of the job.
    private static long stageMinutes = 0;

    // The number of minutes from now when to run the action
    // phase of the job.
    private static long actionMinutes = 0;


    public static void main(String[] args) {
        if (!parseArgs(args)) {
            System.exit(1);
            return;
        }
        try {
            login();
            scheduleRemediationJob();

        } catch (Exception e) {
            // e.printStackTrace();
            System.out.println(e.getMessage());
            System.exit(1);
        }
    } // main

    
    protected static void login() throws Exception {
        // Prompt for the user name and password.
        String[] userPasswd = getUserPassword();

        // Use the OpswareClient to create an https connection to Opsware SAS
        // and authenticate the user.
        try {
           OpswareClient.connect("https", host, (short)port, userPasswd[0],
                                 userPasswd[1], true);
        } catch (com.opsware.fido.AuthenticationException e) {
            System.out.println("Error detected: invalid user/password.");
            System.out.println(e.getMessage());
            System.exit(1);
        } catch (com.opsware.client.ConnectionFailed e) {
            System.out.println("Error detected: Unable to connect to OCC component of Opsware SAS core.");
            System.out.println(e.getMessage());
            System.exit(1);
        }

        // Get the handle to the required service.
        swpSvc = (SoftwarePolicyService)OpswareClient.getService(SoftwarePolicyService.class);
    }


    protected static String[] getUserPassword() throws IOException {
        // Prompt for the user name and password.
        String[] userPasswd = new String[2];
        BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
        System.out.print("Enter Opsware user name: ");
        userPasswd[0] = reader.readLine();

        System.out.print("Enter Opsware user password: ");
        userPasswd[1] = reader.readLine();

        return userPasswd;
    }
    
    protected static boolean parseArgs(String[] args) {
        // Parse the command-line arguments.
        policyID = 0;
        serverID = 0;
        for (int i=0; i < args.length; i++) {

            if ("--host".equals(args[i])) {
                host = args[++i];

            } else if ("--port".equals(args[i])) {
                port = java.lang.Integer.parseInt(args[++i]);

            } else if ("--action".equals(args[i])) {
                actionMinutes = Long.parseLong(args[++i]);

            } else if ("--stage".equals(args[i])) {
                stageMinutes = Long.parseLong(args[++i]);

            } else if ("--policyID".equals(args[i])) {
                policyID = Long.parseLong(args[++i]);

            } else if ("--serverID".equals(args[i])) {
                serverID = Long.parseLong(args[++i]);
            }
        } 

        if (policyID == 0) {
            System.out.println("Missing policy ID.");
            usage();
            return false;
        }

        if (serverID == 0) {
            System.out.println("Missing server ID.");
            usage();
            return false;
        }

        return true;
    } // parseArgs

    
       protected static void usage() {
          StringBuffer sbuf =  new StringBuffer();
          sbuf.append("usage: ScheduleRemediation [options] <required-args>\n");
          sbuf.append("\n");
          sbuf.append("options:\n");
          sbuf.append("         --host <occ host>   OCC host name.\n");
          sbuf.append("         --port <occ port>   OCC port.\n");
          sbuf.append("required-args:\n");
          sbuf.append("         {--stage | --action } <minutes-from-now>\n");
          sbuf.append("         --policyID <ID>\n");
          sbuf.append("         --serverID <ID>\n");
          sbuf.append("\n");
          sbuf.append("Examples:\n");
          sbuf.append("\n");
          sbuf.append("1. Schedule the remediation of the specified policy and server.  The action\n");
          sbuf.append("phase of the remediation will occur 4 minutes from now:\n");
          sbuf.append("   ScheduleRemediation --stage 4 --policyID 566934 --serverID 39567\n");
          sbuf.append("\n");
          sbuf.append("2. Schedule the remediation so that the stage (download) phase is 4 minutes\n");
          sbuf.append("from now and the action phase is 12 minutes from now:\n");
          sbuf.append("   ScheduleRemediation --stage 4 --action 12 --policyID 566934 --serverID 39567\n");
          sbuf.append("\n");
          sbuf.append("3. Specify the host and port of the OCC that the client connects to and then\n");
          sbuf.append("schedule the action phase for 10 minutes from now:\n");
          sbuf.append("   ScheduleRemediation --host occ.c40.dev.opsware.com --port 443 --action 10 --policyID 566934 --serverID 39567\n");
          System.out.println(sbuf.toString());
       } // usage
    
       private static Date dateInMinutesFromNow(long minutes) {
          Date now = new Date();
          long ms = now.getTime() + (minutes * 60 * 1000);
          return new Date(ms);
       } 
    
       private static void printDate(String label, Date date) {
          String formatString = "yyyy.MM.dd.HH.mm.ss";
          Format formatter = new SimpleDateFormat(formatString);
           if (date == null) {
              System.out.println (label + "null");
           }
           else {
              System.out.println(label + formatter.format(date) +
                   " (" + formatString + ")" );
           }
       } 
    
       private static Date fetchHardCodedDate() {
          // Helper class in case you want to hard code the date
          // for testing purposes.
          GregorianCalendar calendar = new GregorianCalendar();
             int year = 2007;
             int month = 10; // January is 0
             int date = 11;
             int hour = 17;
             int minute = 15;
             int second = 0;
             calendar.set(year, month, date, hour, minute, second);
             return(calendar.getTime());
       } 
    

    private static void scheduleRemediationJob() throws Exception {
    
       // The jobArg object is a parameter in the startRemediate method call.
       // Most of the statements in this program are for setting up jobArg.
       PolicyRemediateJobArgument jobArg = new PolicyRemediateJobArgument () ;
    
       // Contained in jobArg, policyAttachableMap specifies servers
       // (or device groups) and policies that are attached.
       PolicyAttachableMap[] policyAttachableMap = new PolicyAttachableMap[1];
       policyAttachableMap [0] = new PolicyAttachableMap();
       PolicyReference[] policies = new PolicyReference[1];
       policies[0] = new SoftwarePolicyRef(policyID);
       policyAttachableMap[0].setPolicies(policies);
       ServerRef[] servers = new ServerRef[1];
       servers[0] = new ServerRef(serverID);
       policyAttachableMap[0].setPolicyAttachables(servers);
    
       //  In the following method call,
       //  set true to remediate with attached policies or
       //  set false to remediate with detached policies.
       policyAttachableMap[0].setAttached(true);
    
       jobArg.setPolicyAttachableMap(policyAttachableMap);
    
       // Contained in jobArg, the globalParams object has parameters that
       // can affect all phases of the job.
       RemediateGlobalParamSet globalParams = new RemediateGlobalParamSet();
       globalParams.setContinueOnFailure(false);
    
       // Allowed values for rebootOption are constants in 
       // RemediateGlobalParamSet:
       // AT_END_REBOOT - Hold all reboots until after all packages are 
       //                 installed or uninstalled.
       // PER_ACTION_REBOOT - Reboot after each action phase.
       // SUPPRESS - Hold all reboots until after all packages are installed
       //            or uninstalled.
       // null - Reboot after each package is installed or uninstalled.
    
       String rebootOption = RemediateGlobalParamSet.SUPPRESS_REBOOT;
       globalParams.setRebootOption(rebootOption);
    
       // If there are no global arguments, you don't need to call the following
       // setGlobalArguments.  This program has global arguments, so the 
       // method is called here:
       jobArg.setGlobalArguments(globalParams);
    
       // A job has three phases: analyze, stage (download), and action.
       // The analyze phase is analagous to clicking the Preview button
       // when you are remediating a policy with the Opsware SAS Client.
       // During the stage phase, the packages to be installed are downloaded
       // from the Software Repository to the managed servers.
       // In the action phase, the actual installation or uninstallation
       // is performed on the managed servers.
       //
       // When you invoke startRemediate, the analyze phase is started
       // immediately.  As of this writing, you cannot schedule the 
       // analyze phase in the API.
       // You can schedule either the stage phase, the action
       // phase, or both.  If you schedule only the stage phase, the action
       // phase begins immediately after the stage phase completes.
       // If you schedule only the action phase, the stage phase begins
       // immediately after the analyze phase.
    
       // The analyzeArgument, stageArgument, and actionArgument objects
       // are also contained in jobArg.  Depending on the command-line
       // argument (selectedPhase), the following lines set the schedule
       // date and time for the phases.
    
       AnalyzeArgument analyzeArgument = new AnalyzeArgument();
       StageArgument stageArgument = new StageArgument();
       ActionArgument actionArgument = new ActionArgument();
    
       printDate("Info: current date (now) = " , new Date());
       Date stageDate = dateInMinutesFromNow(stageMinutes);
       Date actionDate = dateInMinutesFromNow(actionMinutes);
    
       if (stageMinutes != 0) {
          // Schedule the job's stage (download) phase.
          stageArgument.setScheduleDate(stageDate);
          printDate("Info: stageDate = ", stageDate);
       }
       if (actionMinutes != 0) {
          // Schedule the job's action phase.
          actionArgument.setScheduleDate(actionDate);
          printDate("Info: actionDate = ", actionDate);
       }
    
       // Even if you don't invoke set methods on analyzeArgument,
       // stageArgument, and actionArgument, you must set them in
       // jobarg.
       jobArg.setAnalyzePhaseArguments(analyzeArgument);
       jobArg.setStagePhaseArguments(stageArgument);
       jobArg.setActionPhaseArguments(actionArgument);
 
       // Start the remediation job. 
       JobRef jobRef = swpSvc.startRemediate(jobArg);
       System.out.println("Status: Started remediation job.");
       // If you know a job's ID, you can monitor the job's progress 
       // in the SAS Client. 
       System.out.println("Status: Job ID = " + jobRef.getId());
    } 

} 
    
